@@ -1,49 +1,58 @@
 #!/usr/bin/env python
 
-import gtk
-import pango
-import matedesktop
-import gobject
+from gi.repository import Gtk, Gdk, GLib
+from gi.repository import Pango
+from gi.repository import GObject
 import os.path
 import shutil
-import matevfs
 import re
 from execute import *
 import xdg.DesktopEntry
 import xdg.Menu
 from filemonitor import monitor as filemonitor
-import glib
+import ctypes
+from ctypes import *
 
+gtk = CDLL("libgtk-x11-2.0.so.0")
 
-class IconManager(gobject.GObject):
+class IconManager(GObject.GObject):
 
     __gsignals__ = {
-            "changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () )
+            "changed" : (GObject.SignalFlags.RUN_LAST, None, () )
     }
 
     def __init__( self ):
-        gobject.GObject.__init__( self )
+        GObject.GObject.__init__( self )
         self.icons = { }
         self.count = 0
 
         # Some apps don't put a default icon in the default theme folder, so we will search all themes
         def createTheme( d ):
-            theme = gtk.IconTheme()
+            theme = Gtk.IconTheme()
             theme.set_custom_theme( d )
             return theme
 
         # This takes to much time and there are only a very few applications that use icons from different themes
         #self.themes = map(  createTheme, [ d for d in os.listdir( "/usr/share/icons" ) if os.path.isdir( os.path.join( "/usr/share/icons", d ) ) ] )
 
-        defaultTheme = gtk.icon_theme_get_default()
+        self.defaultTheme = Gtk.IconTheme.get_default()
         defaultKdeTheme = createTheme( "kde.default" )
 
+        # Setup and clean up the temp icon dir
+        configDir = GLib.get_user_config_dir()
+        self.iconDir = os.path.join(configDir, "mintmenu")
+        if not os.path.exists(self.iconDir):
+            os.makedirs(self.iconDir)
+        contents = os.listdir(self.iconDir)
+        for fn in contents:
+            os.remove(os.path.join(self.iconDir, fn))
+
+        self.defaultTheme.append_search_path(self.iconDir)
+
         # Themes with the same content as the default them aren't needed
         #self.themes = [ theme for theme in self.themes if  theme.list_icons() != defaultTheme.list_icons() ]
 
-        self.themes = [ defaultTheme, defaultKdeTheme ]
-
-        self.cache = {}
+        self.themes = [ self.defaultTheme, defaultKdeTheme ]
 
         # Listen for changes in the themes
         for theme in self.themes:
@@ -51,93 +60,85 @@ class IconManager(gobject.GObject):
 
 
     def getIcon( self, iconName, iconSize ):
+        
         if not iconName:
             return None
 
         try:
+            iconFileName = ""
+            realIconName = ""
+            needTempFile = False
             #[ iconWidth, iconHeight ] = self.getIconSize( iconSize )
             if iconSize <= 0:
                 return None
 
-            if iconName in self.cache and iconSize in self.cache[iconName]:
-                iconFileName = self.cache[iconName][iconSize]
             elif os.path.isabs( iconName ):
                 iconFileName = iconName
+                needTempFile = True
             else:
                 if iconName[-4:] in [".png", ".xpm", ".svg", ".gif"]:
                     realIconName = iconName[:-4]
                 else:
                     realIconName = iconName
-                tmp = None
-                for theme in self.themes:
-                    if theme.has_icon( realIconName ):
-                        tmp = theme.lookup_icon( realIconName, iconSize, 0 )
-                        if tmp:
-                            break
 
-                if tmp:
-                    iconFileName = tmp.get_filename()
-                else:
-                    iconFileName = ""
+            if iconFileName and needTempFile and os.path.exists( iconFileName ):
+                tmpIconName = iconFileName.replace("/", "-")
+                realIconName = tmpIconName[:-4]
+                if not os.path.exists(os.path.join(self.iconDir, tmpIconName)):
+                    shutil.copyfile(iconFileName, os.path.join(self.iconDir, tmpIconName))
+                    self.defaultTheme.append_search_path(self.iconDir)
 
-            if iconFileName and os.path.exists( iconFileName ):
-                icon = gtk.gdk.pixbuf_new_from_file_at_size( iconFileName, iconSize, iconSize )
+            image = Gtk.Image()
+            icon_found = False
+            for theme in self.themes:
+                if theme.has_icon( realIconName ):
+                    icon_found = True
+                    break
+
+            if icon_found:
+                image.set_from_icon_name(realIconName, Gtk.IconSize.DND)
+                image.set_pixel_size(iconSize)
             else:
-                icon = None
+                image = None
 
-
-            # if the actual icon size is to far from the desired size resize it
-            if icon and (( icon.get_width() - iconSize ) > 5 or ( icon.get_height() - iconSize ) > 5):
-                if icon.get_width() > icon.get_height():
-                    newIcon = icon.scale_simple( iconSize, icon.get_height() * iconSize / icon.get_width(), gtk.gdk.INTERP_BILINEAR )
-                else:
-                    newIcon = icon.scale_simple( icon.get_width() * iconSize / icon.get_height(), iconSize, gtk.gdk.INTERP_BILINEAR )
-                del icon
-                icon = newIcon
-
-            if iconName in self.cache:
-                self.cache[iconName][iconSize] = iconFileName
-            else:
-                self.cache[iconName] = { iconSize : iconFileName }
-
-            return icon
+            return image
         except Exception, e:
             print "Exception " + e.__class__.__name__ + ": " + e.message
             return None
 
     def themeChanged( self, theme ):
-        self.cache = { }
         self.emit( "changed" )
 
-gobject.type_register(IconManager)
+GObject.type_register(IconManager)
 
-class easyButton( gtk.Button ):
+class easyButton( Gtk.Button ):
 
     def __init__( self, iconName, iconSize, labels = None, buttonWidth = -1, buttonHeight = -1 ):
-        gtk.Button.__init__( self )
+        GObject.GObject.__init__( self )
         self.connections = [ ]
         self.iconName = iconName
         self.iconSize = iconSize
         self.showIcon = True
 
-        self.set_relief( gtk.RELIEF_NONE )
+        self.set_relief( Gtk.ReliefStyle.NONE )
         self.set_size_request( buttonWidth, buttonHeight )
 
-        Align1 = gtk.Alignment( 0, 0.5, 1.0, 0 )
-        HBox1 = gtk.HBox()
-        self.labelBox = gtk.VBox( False, 2 )
+        Align1 = Gtk.Alignment.new( 0, 0.5, 1.0, 0 )
+        HBox1 = Gtk.HBox()
+        self.labelBox = Gtk.VBox( False, 2 )
 
 
-        self.buttonImage = gtk.Image()
+        self.buttonImage = Gtk.Image()
         icon = self.getIcon( self.iconSize )
         if icon:
-            self.buttonImage.set_from_pixbuf( icon )
-            del icon
+            self.buttonImage = icon
         else:
             #[ iW, iH ] = iconManager.getIconSize( self.iconSize )
             self.buttonImage.set_size_request( self.iconSize, self.iconSize  )
-        self.buttonImage.show()
-        HBox1.pack_start( self.buttonImage, False, False, 5 )        
+        self.image_box = Gtk.HBox()
+        self.image_box.pack_start(self.buttonImage, False, False, 5)
+        self.image_box.show_all()
+        HBox1.pack_start( self.image_box, False, False, 0 )
 
         if labels:
             for label in labels:
@@ -147,7 +148,7 @@ class easyButton( gtk.Button ):
                     self.addLabel( label[0], label[1] )
 
         self.labelBox.show()
-        HBox1.pack_start( self.labelBox )
+        HBox1.pack_start( self.labelBox , True, True, 0)
         HBox1.show()
         Align1.add( HBox1 )
         Align1.show()
@@ -162,7 +163,7 @@ class easyButton( gtk.Button ):
         self.connections.append( self.connect( event, callback ) )
 
     def onRelease( self, widget ):
-        widget.set_state(gtk.STATE_NORMAL)
+        widget.set_state(Gtk.StateType.NORMAL)
 
     def onDestroy( self, widget ):
         self.buttonImage.clear()
@@ -173,22 +174,22 @@ class easyButton( gtk.Button ):
 
 
     def addLabel( self, text, styles = None ):
-        label = gtk.Label()
-        if "<b>" in text:
+        label = Gtk.Label()
+        if "<b>" in text or "<span" in text:
             label.set_markup(text) # don't remove our pango
         else:
-            label.set_markup(glib.markup_escape_text(text))
+            label.set_text(text)
 
         if styles:
-            labelStyle = pango.AttrList()
+            labelStyle = Pango.AttrList()
             for attr in styles:
                 labelStyle.insert( attr )
             label.set_attributes( labelStyle )
 
-        label.set_ellipsize( pango.ELLIPSIZE_END )
+        label.set_ellipsize( Pango.EllipsizeMode.END )
         label.set_alignment( 0.0, 1.0 )
         label.show()
-        self.labelBox.pack_start( label )
+        self.labelBox.pack_start( label , True, True, 0)
 
 
     def getIcon ( self, iconSize ):
@@ -196,7 +197,7 @@ class easyButton( gtk.Button ):
             return None
 
         icon = iconManager.getIcon( self.iconName, iconSize )
-        if not icon:
+        if icon is None:           
             icon = iconManager.getIcon( "application-default-icon", iconSize )
 
         return icon
@@ -211,11 +212,11 @@ class easyButton( gtk.Button ):
 
     def iconChanged( self ):
         icon = self.getIcon( self.iconSize )
-        self.buttonImage.clear()
+        self.buttonImage.destroy()
         if icon:
-            self.buttonImage.set_from_pixbuf( icon )
-            self.buttonImage.set_size_request( -1, -1 )
-            del icon
+            self.buttonImage = icon
+            self.image_box.pack_start(self.buttonImage, False, False, 5)
+            self.image_box.show_all()
         else:
             #[iW, iH ] = iconManager.getIconSize( self.iconSize )
             self.buttonImage.set_size_request( self.iconSize, self.iconSize  )
@@ -223,15 +224,20 @@ class easyButton( gtk.Button ):
     def setIconSize( self, size ):
         self.iconSize = size
         icon = self.getIcon( self.iconSize )
-        self.buttonImage.clear()
+        self.buttonImage.destroy()
         if icon:
-            self.buttonImage.set_from_pixbuf( icon )
-            self.buttonImage.set_size_request( -1, -1 )
-            del icon
+            self.buttonImage = icon
+            self.image_box.pack_start(self.buttonImage, False, False, 5)
+            self.image_box.show_all()
         elif self.iconSize:
             #[ iW, iH ] = iconManager.getIconSize( self.iconSize )
             self.buttonImage.set_size_request( self.iconSize, self.iconSize  )
 
+class TargetEntry(Structure):
+     _fields_ = [("target", c_char_p),
+                 ("flags", c_int),
+                 ("info", c_int)]
+
 class ApplicationLauncher( easyButton ):
 
     def __init__( self, desktopFile, iconSize):
@@ -257,19 +263,23 @@ class ApplicationLauncher( easyButton ):
 
         base = os.path.basename( self.desktopFile )
         for dir in self.appDirs:
-            self.desktopEntryMonitors.append( filemonitor.addMonitor( os.path.join(dir, base) , self.onDesktopEntryFileChanged ) )
+            self.desktopEntryMonitors.append( filemonitor.addMonitor( os.path.join(dir, base) , self.desktopEntryFileChangedCallback ) )
 
         easyButton.__init__( self, self.appIconName, iconSize )
         self.setupLabels()
 
         # Drag and Drop
-        self.connectSelf( "drag_data_get", self.dragDataGet )
-        self.drag_source_set( gtk.gdk.BUTTON1_MASK  , [ ( "text/plain", 0, 100 ), ( "text/uri-list", 0, 101 ) ], gtk.gdk.ACTION_COPY )
+        self.connectSelf( "drag-data-get", self.dragDataGet )
 
-        icon = self.getIcon( gtk.ICON_SIZE_DND )
+        array = TargetEntry * 2
+        targets = array(( "text/plain", 0, 100 ), ( "text/uri-list", 0, 101 ))
+        gtk.gtk_drag_source_set(hash(self), Gdk.ModifierType.BUTTON1_MASK, targets, 2, Gdk.DragAction.COPY)
+
+        icon = self.getIcon( Gtk.IconSize.DND )
         if icon:
-            self.drag_source_set_icon_pixbuf( icon )
-            del icon
+            iconName, s = icon.get_icon_name()
+            c = c_char_p(iconName.encode('ascii', 'ignore'))
+            gtk.gtk_drag_source_set_icon_name( hash(self), c)
 
         self.connectSelf( "focus-in-event", self.onFocusIn )
         self.connectSelf( "focus-out-event", self.onFocusOut )
@@ -280,14 +290,15 @@ class ApplicationLauncher( easyButton ):
 
     def loadDesktopEntry( self, desktopItem ):
         try:
-            self.appName = desktopItem.getName()
-            self.appGenericName = desktopItem.getGenericName()
-            self.appComment = desktopItem.getComment()
-            self.appExec = desktopItem.getExec()
+            self.appName = self.strip_accents(desktopItem.getName())
+            self.appGenericName = self.strip_accents(desktopItem.getGenericName())
+            self.appComment = self.strip_accents(desktopItem.getComment())
+            self.appExec = self.strip_accents(desktopItem.getExec())
             self.appIconName = desktopItem.getIcon()
             self.appCategories = desktopItem.getCategories()
             self.appGnomeDocPath = desktopItem.get( "X-MATE-DocPath" ) or ""
             self.useTerminal = desktopItem.getTerminal()
+            self.appPath = desktopItem.getPath()
 
             if not self.appGnomeDocPath:
                 self.appKdeDocPath      = desktopItem.getDocPath() or ""
@@ -300,8 +311,8 @@ class ApplicationLauncher( easyButton ):
             self.startupFilePath = os.path.join( os.path.expanduser("~"), ".config", "autostart", basename )
             if self.startupMonitorId:
                 filemonitor.removeMonitor( self.startupMonitorId  )
-            self.startupMonitorId = filemonitor.addMonitor( self.startupFilePath, self.startupFileChanged )
-            #self.inStartup = os.path.exists( self.startupFilePath )
+            if os.path.exists (self.startupFilePath):
+                self.startupMonitorId = filemonitor.addMonitor( self.startupFilePath, self.startupFileChanged )
 
         except Exception, e:
             print e
@@ -316,10 +327,10 @@ class ApplicationLauncher( easyButton ):
 
 
     def onFocusIn( self, widget, event ):
-        self.set_relief( gtk.RELIEF_HALF )
+        self.set_relief( Gtk.ReliefStyle.HALF )
 
     def onFocusOut( self, widget, event ):
-        self.set_relief( gtk.RELIEF_NONE )
+        self.set_relief( Gtk.ReliefStyle.NONE )
 
     def onEnterNotify( self, widget, event ):
         self.grab_focus()
@@ -329,10 +340,10 @@ class ApplicationLauncher( easyButton ):
             
     def filterText( self, text ):
         keywords = text.lower().split()
-        appName = self.strip_accents(self.appName.lower())
-        appGenericName = self.strip_accents(self.appGenericName.lower())
-        appComment = self.strip_accents(self.appComment.lower())
-        appExec = self.strip_accents(self.appExec.lower())
+        appName = self.appName.lower()
+        appGenericName = self.appGenericName.lower()
+        appComment = self.appComment.lower()
+        appExec = self.appExec.lower()
         for keyword in keywords:
             keyw = self.strip_accents(keyword)
             if keyw != "" and appName.find( keyw ) == -1 and appGenericName.find( keyw ) == -1 and appComment.find( keyw ) == -1 and appExec.find( keyw ) == -1:
@@ -342,10 +353,14 @@ class ApplicationLauncher( easyButton ):
         self.show()
         return True
 
-    def strip_accents(self, string):
-        import unicodedata
-        return unicodedata.normalize('NFKD', unicode(string)).encode('UTF8', 'ignore')
-
+    def strip_accents(self, string):        
+        value = string
+        if isinstance(string, unicode):
+            try:
+                value = string.encode('UTF8', 'ignore') 
+            except:
+                pass
+        return value    
 
     def getTooltip( self ):
         tooltip = self.appName
@@ -369,9 +384,11 @@ class ApplicationLauncher( easyButton ):
         if self.appExec:
             if self.useTerminal:
                 cmd = "x-terminal-emulator -e \"" + self.appExec + "\""
-                Execute(cmd)
+                if os.path.exists("/usr/bin/mate-terminal"):
+                    cmd = "mate-terminal -e \"" + self.appExec + "\""
+                Execute(cmd, self.appPath)
             else:
-                Execute(self.appExec)
+                Execute(self.appExec, self.appPath)
 
     def uninstall (self, *args ):
         Execute("gksu /usr/lib/linuxmint/mintMenu/mintRemove.py " + self.desktopFile)
@@ -380,10 +397,11 @@ class ApplicationLauncher( easyButton ):
     def iconChanged( self ):
         easyButton.iconChanged( self )
 
-        icon = self.getIcon( gtk.ICON_SIZE_DND )
+        icon = self.getIcon( Gtk.IconSize.DND )
         if icon:
-            self.drag_source_set_icon_pixbuf( icon )
-            del icon
+            iconName, size = icon.get_icon_name()
+            c = c_char_p(iconName.encode('ascii', 'ignore'))
+            gtk.gtk_drag_source_set_icon_name( hash(self), c)
 
     def startupFileChanged( self, *args ):
         self.inStartup = os.path.exists( self.startupFilePath )
@@ -422,24 +440,6 @@ class ApplicationLauncher( easyButton ):
         #return self.inStartup
         return os.path.exists( self.startupFilePath )
 
-    def hasHelp( self ):
-        return self.appGnomeDocPath or self.appKdeDocPath
-
-    def launchHelp( self ):
-        if self.appGnomeDocPath:
-            bn = os.path.basename( self.appGnomeDocPath )
-            dn = os.path.dirname( self.appGnomeDocPath )
-            if self.appGnomeDocPath[0:6] != "ghelp:":
-                self.appGnomeDocPath = "ghelp:" + self.appGnomeDocPath
-            matevfs.url_show( self.appGnomeDocPath )
-        elif self.appKdeDocPath:
-            if self.appKdeDocPath[0:6] != "help:/" and self.appKdeDocPath[0:6] != "file:/":
-                self.appKdeDocPath = "help:/" + self.appKdeDocPath
-            if self.appKdeDocPath[0:6] == "file:/":
-                matevfs.url_show( self.appKdeDocPath )
-            else:
-                Execute( [ "khelpcenter", self.appKdeDocPath ] )
-
     def onDestroy( self, widget ):
         easyButton.onDestroy( self, widget )
         if self.startupMonitorId:
@@ -447,12 +447,15 @@ class ApplicationLauncher( easyButton ):
         for id in self.desktopEntryMonitors:
             filemonitor.removeMonitor( id )
 
+    def desktopEntryFileChangedCallback (self):
+        GLib.timeout_add(200, self.onDesktopEntryFileChanged)
+
     def onDesktopEntryFileChanged( self ):
         exists = False
         base = os.path.basename( self.desktopFile )
         for dir in self.appDirs:
             if os.path.exists( os.path.join( dir, base ) ):
-                print os.path.join( dir, base ), self.desktopFile
+                # print os.path.join( dir, base ), self.desktopFile
                 self.loadDesktopEntry( xdg.DesktopEntry.DesktopEntry( os.path.join( dir, base ) ) )
                 for child in self.labelBox:
                     child.destroy()
@@ -467,6 +470,7 @@ class ApplicationLauncher( easyButton ):
         if not exists:
             # FIXME: What to do in this case?
             self.destroy()
+        return False
 
 class MenuApplicationLauncher( ApplicationLauncher ):
 
@@ -490,7 +494,7 @@ class MenuApplicationLauncher( ApplicationLauncher ):
         appComment = self.appComment
         if self.highlight: 
             try:
-                #color = self.labelBox.rc_get_style().fg[ gtk.STATE_SELECTED ].to_string()                
+                #color = self.labelBox.rc_get_style().fg[ Gtk.StateType.SELECTED ].to_string()                
                 #if len(color) > 0 and color[0] == "#":
                     #appName = "<span foreground=\"%s\"><b>%s</b></span>" % (color, appName);
                     #appComment = "<span foreground=\"%s\"><b>%s</b></span>" % (color, appComment);
@@ -507,11 +511,11 @@ class MenuApplicationLauncher( ApplicationLauncher ):
         
         if self.showComment and self.appComment != "":
             if self.iconSize <= 2:
-                self.addLabel( appName, [ pango.AttrScale( pango.SCALE_SMALL, 0, -1 ) ] )
-                self.addLabel( appComment, [ pango.AttrScale( pango.SCALE_X_SMALL, 0, -1 ) ] )
+                self.addLabel( '<span size="small">%s</span>' % appName)
+                self.addLabel( '<span size="x-small">%s</span>' % appComment)
             else:
                 self.addLabel( appName )
-                self.addLabel( appComment, [ pango.AttrScale( pango.SCALE_SMALL, 0, -1 ) ] )
+                self.addLabel( '<span size="small">%s</span>' % appComment)
         else:
             self.addLabel( appName )
     
@@ -539,13 +543,13 @@ class FavApplicationLauncher( ApplicationLauncher ):
     def setupLabels( self ):
         if self.appGenericName:
             if self.swapGeneric:
-                self.addLabel( self.appName, [ pango.AttrWeight( pango.WEIGHT_BOLD, 0, -1 ) ] )
+                self.addLabel( '<span weight="bold">%s</span>' % self.appName )
                 self.addLabel( self.appGenericName )
             else:
-                self.addLabel( self.appGenericName, [ pango.AttrWeight( pango.WEIGHT_BOLD, 0, -1 ) ] )
+                self.addLabel( '<span weight="bold">%s</span>' % self.appGenericName )
                 self.addLabel( self.appName )
         else:
-            self.addLabel( self.appName, [ pango.AttrWeight( pango.WEIGHT_BOLD, 0, -1 ) ] )
+            self.addLabel( '<span weight="bold">%s</span>' % self.appName )
             if self.appComment != "":
                 self.addLabel( self.appComment )
             else:
