@@ -1,4 +1,3 @@
-
 import os
 import subprocess
 
@@ -14,17 +13,33 @@ def RemoveArgs(Execline):
 	return NewExecline
 
 # Actually execute the command
-def Execute( cmd ):
-	if isinstance( cmd, str ):
+def Execute( cmd , commandCwd=None):
+	if not commandCwd:
+		cwd = os.path.expanduser( "~" );
+	else:
+		tmpCwd = os.path.expanduser( commandCwd );
+		if (os.path.exists(tmpCwd)):
+			cwd = tmpCwd
+	
+	if isinstance( cmd, str ) or isinstance( cmd, unicode):
 		if (cmd.find("/home/") >= 0) or (cmd.find("su-to-root") >= 0) or (cmd.find("\"") >= 0):
 			print "running manually..."
-			os.system(cmd + " &")
-			return True		
+			try:
+				os.chdir(cwd)
+				os.system(cmd + " &")
+				return True
+			except Exception, detail:
+				print detail
+				return False
 		cmd = cmd.split()
 	cmd = RemoveArgs(cmd)
+	
 	try:
-		os.chdir( os.path.expanduser( "~" ) )
-		subprocess.Popen( cmd )
+		os.chdir( cwd )
+		string = ' '.join(cmd)
+		string = string + " &"
+		#subprocess.Popen( cmd ) // use os.system instead of popen so children don't end up in zombie waiting for us to wait() for them
+		os.system(string)		
 		return True
 	except Exception, detail:
 		print detail
